/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "power.h"

#include <QDebug>
#include "../screen/src/QGSettings/qgsettings.h"
#include <QProcess>
#include <QtDBus/QtDBus>

#define UPOWER_SERVICE         "org.freedesktop.UPower"
#define UPOWER_PATH            "/org/freedesktop/UPower"
#define UPOWER_INTERFACE       "org.freedesktop.UPower"

#define UPOWER_DBUS_PROPERTIES "org.freedesktop.DBus.Properties"
#define UPOWER_DEVICE          "org.freedesktop.UPower.Device"

#define POWERMANAGER_SERVICE   "org.ukui.powerManager"
#define POWERMANAGER_PATH      "/PowerManager"
#define POWERMANAGER_INTERFACE "org.ukui.powerManager"

#define GSETTING_POWERMANAGER  "org.ukui.power-manager"

UkuiPowerInterface::UkuiPowerInterface(QObject *parent) : QObject(parent)
{
    getBatteryDbusPath();

    // 连接设备插入信号
    QDBusConnection::systemBus().connect(
        UPOWER_SERVICE,
        UPOWER_PATH,
        UPOWER_INTERFACE,
        "DeviceAdded",
        this,
        SLOT(onDeviceAdded(QString))
    );

    // 连接设备拔出信号
    QDBusConnection::systemBus().connect(
        UPOWER_SERVICE,
        UPOWER_PATH,
        UPOWER_INTERFACE,
        "DeviceRemoved",
        this,
        SLOT(onDeviceRemoved(QString))
    );
}

UkuiPowerInterface::~UkuiPowerInterface() = default;

static QVariant GetDBusProperty(const QDBusInterface& iface, const QString &propertyName)
{
    if (!iface.isValid()) {
        qDebug() << "QDBusInterface is not valid"<< iface.lastError().message();
        return QVariant();
    }

    QVariant reply = iface.property(propertyName.toUtf8().constData());
    if (!reply.isValid()) {
        qDebug() << "QDBus call failed for service";
    }

    return reply;
}

int UkuiPowerInterface::getDeviceType(const QString &dBusPath)
{
    qDebug() << "device dBus object path:" << dBusPath;

    QDBusInterface dBusInterface(UPOWER_SERVICE,
                                 dBusPath,
                                 UPOWER_DBUS_PROPERTIES,
                                 QDBusConnection::systemBus());
    QDBusReply<QVariant> reply = dBusInterface.call("Get", UPOWER_DEVICE, "Type");
    if (reply.isValid()) {
        return reply.value().toInt();
    } else {
        qDebug() << "Get device type failed";
        return UP_DEVICE_TYPE_UNKNOWN;
    }
}

QVariant UkuiPowerInterface::getBatteryProperty(const QString &propertyName)
{
    QDBusInterface iface(UPOWER_SERVICE,
                         m_batteryPath,
                         UPOWER_DEVICE,
                         QDBusConnection::systemBus());

    return GetDBusProperty(iface, propertyName);
}

void UkuiPowerInterface::getBatteryDbusPath()
{
    QDBusInterface dBusInterface(UPOWER_SERVICE,
                                 UPOWER_PATH,
                                 UPOWER_INTERFACE,
                                 QDBusConnection::systemBus());

    QDBusReply<QList<QDBusObjectPath>> reply = dBusInterface.call("EnumerateDevices");
    if (dBusInterface.isValid()) {
        for (QDBusObjectPath dBusObjectPath : reply.value()) {
            int upDeviceType = getDeviceType(dBusObjectPath.path());
            qDebug() << "device type:" << upDeviceType;
            if (UP_DEVICE_TYPE_BATTERY == upDeviceType) {
                m_batteryPath = dBusObjectPath.path();
                qDebug() << "battery dBusObjectPath:" << m_batteryPath;
            }
        }
    }
}


int UkuiPowerInterface::GetBatteryState()
{
    QVariant reply = getBatteryProperty("State");
    if (!reply.isValid()) {
        sendErrorReply(QDBusError::Failed, "Failed to get Battery State");        
        return -1;
    }

    return reply.toInt();
}

int UkuiPowerInterface::BatteryPercentage()
{
    QVariant reply = getBatteryProperty("Percentage");
    if (!reply.isValid()) {
        sendErrorReply(QDBusError::Failed, "Failed to get Battery Percentage"); 
        return -1;
    }

    return reply.toInt();
}

int UkuiPowerInterface::TimeToFull()
{
    QVariant reply = getBatteryProperty("TimeToFull");
    if (!reply.isValid()) {
        sendErrorReply(QDBusError::Failed, "Failed to get Time To Full");
        return -1;
    }

    return reply.toInt();
}

int UkuiPowerInterface::TimeToEmpty()
{
    QVariant reply = getBatteryProperty("TimeToEmpty");
    if (!reply.isValid()) {
        sendErrorReply(QDBusError::Failed, "Failed to get Time To Empty");
        return -1;
    }

    return reply.toInt();
}

bool UkuiPowerInterface::IsBatteryPresent()
{
    QVariant reply = getBatteryProperty("IsPresent");

    return reply.isValid() && reply.toBool();
}

int UkuiPowerInterface::GetPowerMode()
{   
    QDBusInterface iface(UPOWER_SERVICE,
                         UPOWER_PATH,
                         UPOWER_INTERFACE,
                         QDBusConnection::systemBus());

    QVariant reply = GetDBusProperty(iface, "OnBattery");

    if (!reply.isValid()) {
        return -1;
    }

    if (!QGSettings::isSchemaInstalled(GSETTING_POWERMANAGER)) {
        qWarning() << "org.ukui.power-manager is not install";
        return -1;
    }

    QGSettings settings(GSETTING_POWERMANAGER);
    QVariant value;

    if (reply.toBool()) {
        value = settings.get("power-policy-battery");
        return value.isValid() ? value.toInt() : -1;
    } else {
        value = settings.get("power-policy-ac");
        return value.isValid() ? value.toInt() : -1;
    }
}

void UkuiPowerInterface::SetPowerMode(uint mode)
{
    if (mode > 2)
    {
        qWarning() << "error param " + QString::number(mode) + " just 0 or 1 or 2 can take effect!";
        return;
    }

    QDBusInterface iface(UPOWER_SERVICE,
                         UPOWER_PATH,
                         UPOWER_INTERFACE,
                         QDBusConnection::systemBus());

    QVariant reply = GetDBusProperty(iface, "OnBattery");

    if (!reply.isValid()) {
        return;
    }

    if (!QGSettings::isSchemaInstalled(GSETTING_POWERMANAGER)) {
        qWarning() << "org.ukui.power-manager is not install";
        return;
    }

    QGSettings settings(GSETTING_POWERMANAGER);
    settings.set((reply.toBool() ? "power-policy-battery" : "power-policy-ac"), mode);
    return;
}

int UkuiPowerInterface::GetPowerSaveLevel()
{
    QDBusInterface iface(POWERMANAGER_SERVICE,
                         POWERMANAGER_PATH,
                         POWERMANAGER_INTERFACE,
                         QDBusConnection::sessionBus());

    QDBusReply<int> reply = iface.call("getPowersaveLevel");
    if (!reply.isValid()) {
        qDebug() << "Failed to get powersaveLevel";
        sendErrorReply(QDBusError::Failed, "Failed to get power save Level");
        return -1; // 返回一个表示错误的值
    }

    return reply.value();
}

bool UkuiPowerInterface::IsLid()
{
    QDBusInterface iface(UPOWER_SERVICE,
                         UPOWER_PATH,
                         UPOWER_INTERFACE,
                         QDBusConnection::systemBus());

    QVariant reply = GetDBusProperty(iface, "LidIsPresent");

    return reply.isValid() && reply.toBool();
}

void UkuiPowerInterface::onDeviceAdded(const QString &devicePath)
{
    qDebug() << "Device added:" << devicePath;
    getBatteryDbusPath();
}

void UkuiPowerInterface::onDeviceRemoved(const QString &devicePath)
{
    qDebug() << "Device removed:" << devicePath;
    getBatteryDbusPath();
}
